/*
 * defaults.c
 * 
 * Kevin P. Smith  6/11/89
 * 
 * $Log:$
 * 
 */
#include "copyright2.h"
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <sys/file.h>
#include "Wlib.h"
#include "defs.h"
#include "struct.h"
#include "data.h"

struct stringlist {
   char           *string;
   char           *value;
   struct stringlist *next;
};

struct stringlist *defaults = NULL;

char           *getenv();
char           *strdup();

#ifdef NBTDIST

char           *
new_param(v)

   char		  *v;
{
   char           *temp;

   temp = strdup(v);
   if (strlen(temp) > 12) {
      printf("Distress stat: '%s' is too long. (max = 12)\n", v);
      temp[12] = '\0';
   }
   return temp;
}



figure_distress(file, v)
    char           *file, *v;
{
   int             index = 0;
   char           *check;

   if ((check = strstr(file, "d.sb.")) != NULL)
      index = 1;		/* its a SB param */
   else if ((check = strstr(file, "d.")) == NULL) {
      fprintf(stderr, "Unknown distress param %s\n", file);
      return;
   }
   file = check;
   /* set on/offs */
   if ((check = strstr(file, "whole")) != NULL) {
      distress[index].shld_on = 0;	/* reset the on/offs */
      distress[index].dam_on = 0;
      distress[index].wtmp_on = 0;
      distress[index].etmp_on = 0;
      distress[index].arms_on = 0;
      if (strstr(v, "shld") != NULL)
	 distress[index].shld_on = 1;
      if (strstr(v, "dam") != NULL)
	 distress[index].dam_on = 1;
      if (strstr(v, "wtmp") != NULL)
	 distress[index].wtmp_on = 1;
      if (strstr(v, "etmp") != NULL)
	 distress[index].etmp_on = 1;
      if (strstr(v, "arms") != NULL)
	 distress[index].arms_on = 1;
   }
   /* Do the shields */
   else if ((check = strstr(file, "shld")) != NULL) {
      if (strstr(check, "lvl"))
	 sscanf(v, "%d %d", &distress[index].min_shld, &distress[index].max_shld);
      else if (strstr(check, "low"))
	 distress[index].low_shld = new_param(v);
      else if (strstr(check, "mid"))
	 distress[index].mid_shld = new_param(v);
      else if (strstr(check, "high"))
	 distress[index].high_shld = new_param(v);
   }
   /* Do the Damage */
   else if ((check = strstr(file, "dam")) != NULL) {
      if (strstr(check, "lvl"))
	 sscanf(v, "%d %d", &distress[index].min_dam, &distress[index].max_dam);
      else if (strstr(check, "low"))
	 distress[index].low_dam = new_param(v);
      else if (strstr(check, "mid"))
	 distress[index].mid_dam = new_param(v);
      else if (strstr(check, "high"))
	 distress[index].high_dam = new_param(v);
   }
   /* Do the wtmp */
   else if ((check = strstr(file, "wtmp")) != NULL) {
      if (strstr(check, "lvl"))
	 sscanf(v, "%d %d", &distress[index].min_wtmp, &distress[index].max_wtmp);
      else if (strstr(check, "low"))
	 distress[index].low_wtmp = new_param(v);
      else if (strstr(check, "mid"))
	 distress[index].mid_wtmp = new_param(v);
      else if (strstr(check, "high"))
	 distress[index].high_wtmp = new_param(v);
   }
   /* Do the etmp */
   else if ((check = strstr(file, "etmp")) != NULL) {
      if (strstr(check, "lvl"))
	 sscanf(v, "%d %d", &distress[index].min_etmp, &distress[index].max_etmp);
      else if (strstr(check, "low"))
	 distress[index].low_etmp = new_param(v);
      else if (strstr(check, "mid"))
	 distress[index].mid_etmp = new_param(v);
      else if (strstr(check, "high"))
	 distress[index].high_etmp = new_param(v);
   }
   /* Do the armies */
   else if ((check = strstr(file, "arms")) != NULL) {
      if (strstr(check, "lvl"))
	 sscanf(v, "%d %d", &distress[index].min_arms, &distress[index].max_arms);
      else if (strstr(check, "low"))
	 distress[index].low_arms = new_param(v);
      else if (strstr(check, "mid"))
	 distress[index].mid_arms = new_param(v);
      else if (strstr(check, "high"))
	 distress[index].high_arms = new_param(v);
   }
   /* Do the fuel */
   else if ((check = strstr(file, "fuel")) != NULL) {
      if (strstr(check, "lvl"))
	 sscanf(v, "%d %d", &distress[index].min_fuel, &distress[index].max_fuel);
      else if (strstr(check, "low"))
	 distress[index].low_fuel = new_param(v);
      else if (strstr(check, "mid"))
	 distress[index].mid_fuel = new_param(v);
      else if (strstr(check, "high"))
	 distress[index].high_fuel = new_param(v);
   }
   /* What else is there except an error */
   else
      fprintf(stderr, "Unknown distress param %s\n", file);
}

#endif				/* NBTDIST*/


initDefaults(deffile)
    char           *deffile;	/* As opposed to defile? */
{
   FILE           *fp;
   char            file[256];
   char           *home;
   char           *v;
   struct stringlist *new;

   if (!deffile) {
      deffile = file;
      home = getenv("HOME");
      if (!findDefaults(home, /* initialized */ deffile)) {
	 if (home) {
	    sprintf(file, "%s/.xtrekrc", home);
	 } else {
	    strcpy(file, ".xtrekrc");
	 }
	 deffile = file;
      }
   }
   fp = fopen(deffile, "r");
   if (!fp)
      return;
   IFDEBUG(printf("Using defaults file %s\n", deffile);)
#ifdef NBT
   macrocnt=0; /* reset macros */
#endif
   while (fgets(file, 99, fp)) {
      if (*file == '#')
	 continue;
      if (*file != 0)
	 file[strlen(file) - 1] = '\0';
      v = file;
      while (*v != ':' && *v != 0) {
	 v++;
      }
      if (*v == 0)
	 continue;
      *v = 0;
      v++;
      while (*v == ' ' || *v == '\t') {
	 v++;
      }
#ifdef NBT
      if (strncmp(file, "macro.", 6) == 0) {
	 if (macrocnt == MAX_MACRO) {
	    fprintf(stderr, "Maximum number of macros is %d\n", MAX_MACRO);
	 }
	 else if (file[6]=='?'){
	     fprintf(stderr,"Cannot use '?' for a macro\n");
	 }
	 else {
#ifdef FEATURE
            macro[macrocnt].type=NBTM;
#endif
	    macro[macrocnt].key = file[6];
	    macro[macrocnt].who = file[8];
	    macro[macrocnt].string = strdup(v);
	    macrocnt++;
	 }
      } else 
#endif

#ifdef FEATURE
      if (strncmp(file,"mac.",4)==0) {
	 if (macrocnt==MAX_MACRO) {
	    fprintf(stderr,"Maximum number of macros is %d\n",MAX_MACRO);
	 }
	 else if (file[4]=='?')
	    fprintf(stderr,"Cannot use '?' for a macro\n");
	 else  {
	    macro[macrocnt].key=file[4];

            if (file[5] == '.') {
	       if (file[6] == '%') {
		  switch (file[7]) {
		     case 'u':
                     case 'U':
			macro[macrocnt].who=MACRO_PLAYER;
                        break;
                     case 't':
                     case 'T':
			macro[macrocnt].who=MACRO_TEAM;
                        break;
                     default:
			macro[macrocnt].who=MACRO_ME;
                        break;
		  }
                  macro[macrocnt].type=NEWMMOUSE;
	       }
               else {
		  macro[macrocnt].who=file[6];
                  macro[macrocnt].type=NEWMSPEC;
               }
	    }
            else {
	       macro[macrocnt].who='\0';
               macro[macrocnt].type=NEWM;
            }
            macro[macrocnt].string=strdup(v);
            macrocnt++;
	 }
      } else
#endif

#ifdef NBTDIST
      if (strncmp(file, "d.", 2) == 0)
	 figure_distress(file, v);
      else
#endif
      if (*v != 0) {
	 new = (struct stringlist *) malloc(sizeof(struct stringlist));
	 new->next = defaults;
	 new->string = strdup(file);
	 new->value = strdup(v);
	 defaults = new;
      }
   }
   fclose(fp);
}

/*
 * free up the defaults linked list, so that it can possibly be * re-read, or
 * for Possible cases where exit doesn't clear allocated * memory (for the
 * eventual Amiga port!)   -EM 12/3/92
 */
int 
FreeDefaults()
{
   struct stringlist *freeme;
   struct stringlist *nextnode;
   nextnode = defaults;
   while (nextnode != NULL) {
      free(nextnode->value);
      free(nextnode->string);
      freeme = nextnode;
      nextnode = freeme->next;
      free(freeme);
   }
}

char           *
strdup(str)
#if defined(__STDC__)		/* doing this so strdup() will match
				   prototype in string.h */
const
#endif
    char           *str;
{
   char           *s;

   s = (char *) malloc(strlen(str) + 1);
   strcpy(s, str);
   return (s);
}

char           *
getdefault(str)
    char           *str;
{
   struct stringlist *sl;

   /*
   printf("\"%s\"\n", str);
   */
   sl = defaults;
   while (sl != NULL) {
      if (strcmpi(sl->string, str) == 0) {
	 return (sl->value);
      }
      sl = sl->next;
   }
   return (NULL);
}

/*
 * strcmpi tweaked 9/17/92 E-Mehlhaff to not tweak the strings it's * called
 * with... * writing into a 'constant' strings space is bad!
 */
strcmpi(str1, str2)
    char           *str1, *str2;
{
   char            chr1, chr2;
   /* #ifdef strcasecmp */
   return (strcasecmp(str1, str2));
   /*
    * #else for(;;) { chr1 = isupper(*str1) ? *str1 : toupper(*str1); chr2 =
    * isupper(*str2) ? *str2 : toupper(*str2); if (chr1 != chr2) return(chr2
    * - chr1); if (chr1==0 || chr2==0) return(1); if (chr1==0 && chr2==0)
    * return(0); str1++; str2++; } return (0); #endif
    */
}

booleanDefault(def, preferred)
    char           *def;
    int             preferred;
{
   char           *str;

   str = getdefault(def);
   if (str == NULL)
      return (preferred);
   if (strcmpi(str, "on") == 0) {
      return (1);
   } else {
      return (0);
   }
}

#ifdef TSH
intDefault(def, preferred)
    char           *def;
    int             preferred;
{
   char           *str;
   str = getdefault(def);
   if (!str)
      return preferred;
   return atoi(str);
}

/*
 * no default file given on command line. See if serverName is defined.  If
 * it exists we look for HOME/.xtrekrc-<serverName> and .xtrekrc-<serverName>
 * Otherwise we try DEFAULT_SERVER.
 */

/*
 * since this is Find Defaults, I moved all the defaults file checking to *
 * it, and put in support for a system defaults file. * and it uses the
 * access() system call to determine if a defaults *  file exists. * note,
 * access() returns 0 if user can read file, -1 on error or if * they can't. *
 * -EM *
 * 
 * Is anyone else bothered by the fact that this writes to deffile * without
 * really knowing how much of deffile is allocated? *
 * 
 */

findDefaults(home, deffile)
    char           *home, *deffile;
{
   int             accessible = -1;

   /* first look for server specific .xtrekrc in $HOME */
   if (serverName) {
      if (home)
	 sprintf(deffile, "%s/.xtrekrc-%s", home, serverName);
      else
	 sprintf(deffile, ".xtrekrc-%s", serverName);
      IFDEBUG(printf("Looking for defaults file in %s\n", deffile);)
      accessible = access(deffile, R_OK);
      if (accessible == 0)
	 return 1;
      /* else return 0; */
   }
   /* then try default server server-speciric defaults file in $HOME */
   if (home)
      sprintf(deffile, "%s/.xtrekrc-%s", home, DEFAULT_SERVER);
   else
      sprintf(deffile, ".xtrekrc-%s", DEFAULT_SERVER);
   IFDEBUG(printf("Looking for defaults file in %s\n", deffile);)
   accessible = access(deffile, R_OK);
   if (accessible == 0)
      return 1;

   /* now try .xtrekrc in $HOME */
   if (home)
      sprintf(deffile, "%s/.xtrekrc", home);
   else
      sprintf(deffile, ".xtrekrc");
   IFDEBUG(printf("Looking for defaults file in %s\n", deffile);)
   accessible = access(deffile, R_OK);
   if (accessible == 0)
      return 1;

   /* and of course, try for a .netrekrc  in $HOME, for the new generation */
   /* now try .xtrekrc in $HOME */
   if (home)
      sprintf(deffile, "%s/.netrekrc", home);
   else
      sprintf(deffile, ".netrekrc");
   IFDEBUG(printf("Looking for defaults file in %s\n", deffile);)
   accessible = access(deffile, R_OK);
   if (accessible == 0)
      return 1;

#ifdef SYSTEM_DEFAULTFILE
   /* now try for a system default defaults file */
   sprintf(deffile, SYSTEM_DEFAULTFILE);
   IFDEBUG(printf("Looking for defaults file in %s\n", deffile);)
   accessible = access(deffile, R_OK);
   if (accessible == 0)
      return 1;
#endif
   return 0;
}

#endif				/* TSH */

#ifdef MOO
int 
defaultShip(preferred)
    int             preferred;
{
   char           *type;

   type = getdefault("defaultShip");
   if (type == NULL)
      return preferred;
   if ((strcmpi(type, "scout") == 0) || (strcmpi(type, "SC") == 0))
      return SCOUT;
   else if ((strcmpi(type, "destroyer") == 0) || (strcmpi(type, "DD") == 0))
      return DESTROYER;
   else if ((strcmpi(type, "cruiser") == 0) || (strcmpi(type, "CA") == 0))
      return CRUISER;
   else if ((strcmpi(type, "battleship") == 0) || (strcmpi(type, "BB") == 0))
      return BATTLESHIP;
   else if ((strcmpi(type, "assault") == 0) || (strcmpi(type, "AS") == 0))
      return ASSAULT;
   else if ((strcmpi(type, "starbase") == 0) || (strcmpi(type, "SB") == 0))
      return STARBASE;
   else
      return preferred;
}
#endif
